﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices; // run
using System.Reflection;
using Microsoft.Win32;

using System.Windows.Threading;  // Timer


namespace WPFEditor
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        // search
        private String search = "";
        private Boolean ignoreCase = false;
        private int position = 0;

        String sFilename = "";
        int fontsize = 10;
        private Boolean modified = false;

        public MainWindow()
        {
            InitializeComponent();
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            fontsize = 10;
            mnFont10pg.IsChecked = true;

            String[] args = System.Environment.GetCommandLineArgs();
            for (int i = 1; i < args.Length; i++)
            {
                String file = args[i];
                String sExt = basis.getFilenameExt(file).ToLower();
                sFilename = file;
                loadFilename(sFilename);
                this.Title = sFilename;
                 break;  // nur der erste gueltige Parameter
            }
            // multiLine
            this.editor.TextWrapping = TextWrapping.Wrap;
            this.editor.AcceptsReturn = true;
            this.editor.AcceptsTab = true;
            this.editor.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;
            this.editor.VerticalScrollBarVisibility = ScrollBarVisibility.Visible;

            fontsize = 10;
            CommandBindings.Add(new CommandBinding(MyCommands.Newfile, NewExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Openfile, OpenExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Savefile, SaveExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SaveAsfile, SaveAsExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Undo, UndoExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Redo, RedoExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Copy, CopyExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Paste, PasteExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Cut, CutExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Selectall, SelectallExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Wordwrap, WordwrapExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Search, SearchExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SearchAgain, SearchAgainExecuted, CheckCanExecuted));
                    
            CommandBindings.Add(new CommandBinding(MyCommands.NewInstanz, NewInstanzExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Close, CloseExecuted, CheckCanExecuted));

            DispatcherTimer dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            // dispatcherTimer.Interval = new TimeSpan(0, 0, 1);  // hour, min, sec, ms
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(250);
            
            dispatcherTimer.Start();
            lDatei.Content = "";
            this.editor.TextChanged += textBox1_TextChanged;
        }  // window_Loaded

        private void dispatcherTimer_Tick(object sender, EventArgs e)
        {            
            setLineNummer();
            CommandManager.InvalidateRequerySuggested();
        }

        public void setLineNummer()
        {
            int charIndex = editor.CaretIndex;
            int lineindex = (editor.GetLineIndexFromCharacterIndex(charIndex) + 1);
            int index_aktLine = editor.GetCharacterIndexFromLineIndex(lineindex - 1);
            this.lRowNr.Content = "Zl: " + lineindex;
            int columnCount = editor.SelectionStart - index_aktLine + 1;
            this.lColumnNr.Content = "Sp: " + columnCount;
        }

        private void textBox1_TextChanged(object sender, TextChangedEventArgs e)
        {
            modified = true;
        }

        private void checkFile()
        {
            if (modified)
            {
                MessageBoxResult res = basis.readYesNoCancel("Sollen die Änderungen im Editor gespeichert werden?", "Datei laden");
                switch (res)
                {
                    case MessageBoxResult.Yes: saveFile();
                        break;
                    case MessageBoxResult.No:
                        break;
                    case MessageBoxResult.Cancel:
                        break;
                }
            }
        }

        private void loadFilename(String sFilename)
        {
            checkFile();
            string sStr;
            FileStream oFile = new FileStream(sFilename, FileMode.Open);
            StreamReader inStream = new StreamReader(oFile);
            sStr = inStream.ReadToEnd();
            editor.Text = sStr;
            inStream.Close();
            oFile.Close();
            modified = false;
            editor.SelectionStart = 0;  // set the cursor to the begin of the text
            editor.SelectionLength = 0;
            lDatei.Content = sFilename;
            Title = "WPF-Editor: " + sFilename;
        }

        private void saveFile()
        {
            FileStream oFile;
            StreamWriter oStream;
            if (!sFilename.Equals(""))
            {
                oFile = new FileStream(sFilename, FileMode.Create);
                oStream = new StreamWriter(oFile);
                oStream.WriteLine(editor.Text);
                oStream.Close();
                oFile.Close();
            }
            modified = false;
        }  // saveFile


        private void CheckCanExecuted(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
        }

        private void NewExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            sFilename = "";
            editor.Text = "";
            modified = false;
        }  // mnNew_Click

        private void OpenExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Text-Dateien (*.txt,dat)|*.txt;*.dat|XML-Dateien (*.xml)|*.xml|Alle Dateien (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;// zählt von 1 !
            openFileDialog1.DefaultExt = ".txt";
            openFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == true)
            {
                sFilename = openFileDialog1.FileName.ToString();
                loadFilename(sFilename);
                this.Title = sFilename;
            }
        }

        private void SaveExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            if (sFilename.Equals(""))
            {
                SaveAsExecuted(sender, e);
            }
            else
            {
                saveFile();
            }
        }

        private void SaveAsExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Title = "Speichern unter";
            saveFileDialog1.Filter = "Text-Dateien (*.txt,dat)|*.txt;*.dat|XML-Dateien (*.xml)|*.xml|Alle Dateien (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".txt";
            saveFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            saveFileDialog1.RestoreDirectory = true;
            if (saveFileDialog1.ShowDialog() == true)
            {
                sFilename = saveFileDialog1.FileName.ToString();
                this.Title = sFilename;
                saveFile();                
            }
        }  // mnSaveAs_Click

        private void UndoExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            editor.Undo();
        }
        private void RedoExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            editor.Redo();
        }

        private void CopyExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            editor.Copy();
        }

        private void PasteExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            editor.Paste();
        }
        private void CutExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            editor.Cut();
        }

        private void SelectallExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            editor.SelectAll();
        }

        private void SearchExecuted(object sender, ExecutedRoutedEventArgs e)
        {           
            FSearch dialog = new FSearch(this);
            dialog.ShowDialog();
            if (dialog.Result)
            {
                search = dialog.SearchString;
                ignoreCase = dialog.IgnoreCase;
                searchText(search, ignoreCase);
            }
        }

        private void SearchAgainExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            searchAgain();
        }


        public void searchText(String search, Boolean ignoreCase)
        {
            this.search = search;
            this.ignoreCase = ignoreCase;
            position = 0;
            int index;
            if (this.ignoreCase)
                index = editor.Text.IndexOf(search, position, StringComparison.OrdinalIgnoreCase);
            else
                index = editor.Text.IndexOf(search, position);
            position = index + 1;
            if (index > 0)
            {
                editor.Select(index, search.Length);
                editor.SelectionStart = index;
            }
            else
            {
                MessageBox.Show("Die Suche war nicht erfolgreich.\r\nSuchwert: " + search,
                    "Suchen", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
        }

        public void searchAgain()
        {
            int index;
            if (this.ignoreCase)
                index = editor.Text.IndexOf(search, position, StringComparison.OrdinalIgnoreCase);
            else
                index = editor.Text.IndexOf(search, position);
            position = index + 1;
            if (index > 0)
            {
                editor.Select(index, search.Length);
                editor.SelectionStart = index;               
            }
        }  // searchAgain



        private void NewInstanzExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "WPFEditor.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start(); 

        }
        

        private void WordwrapExecuted(object sender, ExecutedRoutedEventArgs e)
       {
           SingVar singvar = SingVar.getInstance();
           mnWordWrap.IsChecked = !mnWordWrap.IsChecked;
           singvar.WordWrap = mnWordWrap.IsChecked;
           if (singvar.WordWrap)
               editor.TextWrapping = TextWrapping.Wrap;
           else
              editor.TextWrapping = TextWrapping.NoWrap;           
       }

        private void clearFontMenues(object sender)
        {
            mnFont8pg.IsChecked = false;
            mnFont10pg.IsChecked = false;
            mnFont12pg.IsChecked = false;
            mnFont14pg.IsChecked = false;
            mnFont16pg.IsChecked = false;
            mnFont18pg.IsChecked = false;
            mnFont20pg.IsChecked = false;
            mnFont24pg.IsChecked = false;
            mnFont28pg.IsChecked = false;
            MenuItem mnItem = (MenuItem)sender;
            mnItem.IsChecked = true;
        }

        private void mnFont_Click(object sender, RoutedEventArgs e)
        {
            MenuItem mn = (MenuItem)sender;
            clearFontMenues(sender);
            setfontsize(Convert.ToInt32(mn.Tag));
        }

        private void setfontsize(int size)
        {
            this.fontsize = size;
            editor.FontSize = size;
            lRowNr.FontSize = size;
            lColumnNr.FontSize = size;
            lDatei.FontSize = size;
            toolbar1.FontSize = size;
            mainMenu.FontSize = size;
        }



        private void CloseExecuted(object sender, ExecutedRoutedEventArgs e)
        {
            Close();
        }



        private void Window_Closed(object sender, EventArgs e)
        {
           SingVar singvar = SingVar.getInstance();           
            singvar.Setting = true;
            singvar.Top = (int)Top;
            singvar.Left = (int)Left;
            singvar.Width = (int)Width;
            singvar.Height = (int)Height;
        }  // Window_Closed

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {            
            if (modified)
            {
                MessageBoxResult res = basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Programm beenden");
                switch (res)
                {
                    case MessageBoxResult.Yes: saveFile();
                        break;
                    case MessageBoxResult.No:
                        break;
                    case MessageBoxResult.Cancel: e.Cancel = true;
                        break;
                }
            }
        } // Window_Closing


    }
}
